/** @file eeprom.c

   This file is taken from the Windows driver and adapted for Linux.

   @par CREATION
   @author Krauth Julien
   @date   09.10.2006

   @par VERSION
   @verbatim
   $LastChangedRevision:$
   $LastChangedDate:$
   @endverbatim

   @par LICENCE
   @verbatim
   Copyright (C) 2004,2005  ADDI-DATA GmbH for the source code of this module.

        ADDI-DATA GmbH
        Dieselstrasse 3
        D-77833 Ottersweier
        Tel: +19(0)7223/9493-0
        Fax: +49(0)7223/9493-92
        http://www.addi-data-com
        info@addi-data.com
   @endverbatim
 */

/*  +-----------------------------------------------------------------------+  */
/*  | (C) ADDI-DATA GmbH          Dieselstrasse 3      D-77833 Ottersweier  |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Tel : +49 (0) 7223/9493-0     | email    : info@addi-data.com         |  */
/*  | Fax : +49 (0) 7223/9493-92    | Internet : http:$/$/www.addi-data.com   |*/
/*  +-----------------------------------------------------------------------+  */
/*  | Project   : -                 | Compiler : BORLAND C                  |  */
/*  | Modulname : Eeprom.c          | Version  : 3.1                        |  */
/*  +-------------------------------+---------------------------------------+  */
/*  | Author    : S.Weber           | Date     : 26.02.04                   |  */
/*  +-----------------------------------------------------------------------+  */
/*  | Description : Eeprom management                                       |  */
/*  +-----------------------------------------------------------------------+  */
/*  |                             UPDATE'S                                  |  */
/*  +-----------------------------------------------------------------------+  */
/*  |   Date   |   Author  |          Description of updates                |  */
/*  +----------+-----------+------------------------------------------------+  */
/*  |          |           |                                                |  */
/*  +-----------------------------------------------------------------------+  */
/* Include*/
/*+--------------------------------------------------------------------------------------------+*/
/*| INCLUDE                                                                                    |*/
/*+--------------------------------------------------------------------------------------------+*/



#include "../xpci3xxx-private.h"

#include "include/eeprom.h"

//-----------------------------------------------------------------------------

static int check_EepromRead(uint32_t PCIBoardEepromAddress)
{
  uint32_t dw_ReadAddress;
  INPDW (PCIBoardEepromAddress, &dw_ReadAddress);
	return ((dw_ReadAddress & 1) == 0);
}

//-----------------------------------------------------------------------------

/** busy loops until the condition is true or a timeout occurs
 *
 * This function is intended to be used in context when interrupts are or could be deactivated.
 *
 * Its use must be restrained to these cases, as busylooping is the most inefficient way of doing that.
 *
 * @param[in] pdev The device concerned.
 * @param[in] checkf A function that performs the check. it must return 0 if the condition is not fullfilled, 1 otherwise
 * @param[in] timeout The timeout in ticks of 5 microseconds
 *
 */
static inline int xpci3xxx_eeprom_busywait_with_timeout(int (*checkf) (uint32_t PCIBoardEepromAddress), uint32_t PCIBoardEepromAddress, unsigned int timeout)
{
    unsigned long count = 0;
    while(1)
    {
    if (checkf (PCIBoardEepromAddress))
          return 0; // condition becomes true

        // wait 100 microseconds
        udelay(100);
        count++;
        if (count>=timeout)
          return 1; // timeout
    }
}

//-----------------------------------------------------------------------------

/*i_EepromRead*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : int  i_EepromRead    (pstr_AddressInformation ps_Address,   |*/
/*|                                           uint16_t                w_offset,     |*/
/*|                                           uint16_t *              pw_Value)     |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function read a value from the EEPROM 93C76.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address : PCI eeprom base address                        |*/
/*|                     w_offset   : Offset of the adress to read                   |*/
/*|                     pw_Value   : PCI eeprom 16 bit read value.                  |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
int  i_EepromRead    (uint32_t    PCIBoardEepromAddress,
                      uint16_t    w_offset,
                      uint16_t *  pw_Value)
      {
        uint32_t  dw_RegisterValue     = 0;

        /*******************************/
        /* Set the eeprom read address */
        /*******************************/
        dw_RegisterValue = w_offset / 2;
        dw_RegisterValue = dw_RegisterValue << 16;
        OUTPDW (PCIBoardEepromAddress + 0x04, dw_RegisterValue);

        /************************/
        /* Set the read command */
        /************************/
        OUTPDW (PCIBoardEepromAddress, 0x8);

        /* timeout 1 second */
       	if ( xpci3xxx_eeprom_busywait_with_timeout(check_EepromRead, PCIBoardEepromAddress, 10000) )
       		return -1;

      	INPDW (PCIBoardEepromAddress + 0x04, &dw_RegisterValue);

      	*pw_Value = (unsigned short) (dw_RegisterValue & 0xFFFF);

        return (0);
      }

/*i_EepromReadBlock*/
/*+---------------------------------------------------------------------------------+*/
/*| Function   Name   : int  i_EepromReadBlock                                      |*/
/*                                    (pstr_AddressInformation ps_Address,          |*/
/*|                                    uint16_t                w_StartAddress,      |*/
/*|                                    uint16_t                w_Size,              |*/
/*|                                    uint16_t *              pw_Value)            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Task              : This function read a block from the EEPROM 93C76.           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Input Parameters  : ps_Address       : PCI eeprom base address                  |*/
/*|                     w_StartAddress   : Start address                            |*/
/*|                     pw_Value         : PCI eeprom 16 bit read block.            |*/
/*+---------------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                           |*/
/*+---------------------------------------------------------------------------------+*/
/*| Return Value      : 0 : OK                                                      |*/
/*|                    -1 : time out                                                |*/
/*+---------------------------------------------------------------------------------+*/
int  i_EepromReadBlock (uint16_t	w_Size,
			uint32_t	PCIBoardEepromAddress,
			uint16_t	w_StartAddress,
			uint16_t *	pw_Value)
      {
        /************************/
        /* Variable declaration */
        /************************/
        int       i_Error           = 0;
        uint16_t  w_AddressCounter  = 0;

        for (w_AddressCounter = 0;
             (w_AddressCounter < w_Size) && (i_Error == 0);
             w_AddressCounter ++, pw_Value ++)
        {
          /*******************/
          /* Read the Eeprom */
          /*******************/
          i_Error = i_EepromRead (PCIBoardEepromAddress, (uint16_t) (w_StartAddress + 2*w_AddressCounter), pw_Value);
        }
        return (i_Error);
      }
